import java.util.*;

public class GameServer extends Thread {
	private Vector clients = new Vector ();
	private int clientLimit;
	private Gamestate gamestate;
	private boolean runServer = false;

    public GameServer(Gamestate s, int maxClients) {
        gamestate = s;
		clientLimit = maxClients;
		runServer = true;
    }

	public void stopServer () {
		runServer = false;
	}

	public void disconnectClient (ClientHandler c) {
		// Eemaldab kliendi
		if (clients.contains (c)) {
			System.out.println ("Removing client for player " + c.getPlayer ().name );
			synchronized (gamestate.getPlayerList ()) {
				gamestate.getPlayerList ().remove (c.getPlayer ());
			}
			clients.remove (c);
		}
	}

    public void run() {
		System.out.println ("Waiting for clients...");
		while (runServer) {

			synchronized (gamestate) {
				if (gamestate.shutdownOrder) {
					System.out.println ("Shutting down server...");
					for (int i = 0; i < clients.size (); i++) {
						disconnectClient ((ClientHandler)clients.elementAt (i));
					}
					runServer = false;
					break;
				}
			}

			try {
				sleep((int)(Math.random() * 300));
			} catch (InterruptedException e) {}

			// Suppose there is someone at the socket...
			System.out.println ("Client trying to connect!");

			if (clients.size () == clientLimit) {
				System.out.println ("Dropping client, because server is full!");
			} else {

				synchronized (gamestate) {

					if (!gamestate.shutdownOrder) {
						// Create a new thread to take care of the client
						Player p = new Player ();
						p.type = Player.NETWORK;
						
						// We don't want to upset other threads
						synchronized (gamestate.getPlayerList ()) {
							gamestate.getPlayerList ().add (p);
						}

						ClientHandler ch = new ClientHandler (this, gamestate, p);
						clients.add (ch);
						ch.start ();

					}
				}
			}
		}
    }
}

