import java.util.*;

/* This class stores the game state and data (like players etc) */

public class Gamestate {

	// Some constants
    public static final int STOPPED = 0;
    public static final int AI = 1;
    public static final int PHYSICS = 2;
	public static final int RENDERING = 3;
	
	public static final int FIRST = AI;
	public static final int LAST = RENDERING;

	// Running status
	public boolean gameRunning = false;
	public boolean shutdownOrder = false;
	
	// Frame status
	private boolean processing = false;	
	private int gamestate;

	// The players of the game and their game data
	private Vector players = new Vector ();

	// Starts the game
	public void runGame () {
		gameRunning = true;
		gamestate = FIRST;
	}
	
	// Schedules the stopping of the game
	public void stopGame () {
		synchronized (this) {
			shutdownOrder = true;
		}
	}

	// A frame processing thread reserves the gamestate for processing
	public synchronized void processState () {
		if (processing) {
            try {
                wait();
            } catch (InterruptedException e) { }
		}
		
		// Here the processing thread is given the data or something
		processing = true;
	}

	// The processing thread releases the gamestate
	public synchronized void advanceState () {
		processing = false;

		if (gamestate == LAST) {
			synchronized (this) {
				if (shutdownOrder == true) {
					gameRunning = false;
					gamestate = STOPPED;
				} else {
					gamestate = FIRST;
				}
			}
		} else
			gamestate++;
			
		notifyAll ();
	}
	
	// Returns the state
    public synchronized int getState() {
        return gamestate;
    }

	public Vector getPlayerList () {
		return players;
	}

}
