// Using the Apache XML-RPC implementation version 2
import org.apache.xmlrpc.*;
import java.util.Vector;
import java.io.*;

public class MasterClient {

	// The RPC client
	private static XmlRpcClient client;
	
	// The local information about the client
	private String servername;
	private String serveraddress;
	
	public static void main (String args[]) throws Exception {

		if (args.length < 1) {
			System.out.println ("Usage: java MasterClient MasterServerURL");
		} else {
			
			BufferedReader inp = new BufferedReader(new InputStreamReader(System.in));
			System.out.println ("Starting master client...");

			try {
				
				// Get server information for the server
				System.out.print ("Enter server name: ");
				String name = inp.readLine();
				
				System.out.print ("Enter server address: ");
				String address = inp.readLine ();
			
				// Create the local client instance
				MasterClient client = new MasterClient (args[0], name, address);
				client.run ();
			
			} catch (Exception e) {
				System.out.println ("Error setting up the server!");
			}
		}
		
	}

	// Set up the client
	public MasterClient (String url, String name, String address) throws Exception {
		client = new XmlRpcClient (url);
		servername = name;
		serveraddress = address;
	}
	
	// The client main loop
	public void run () throws IOException {
		
		try {

			// Register the client at the master server
			Vector parameters = new Vector ();
			parameters.add (servername);
			parameters.add (serveraddress);
			
			Integer assignedID = (Integer)client.execute ("registerServer", parameters);
			System.out.println ("Master server assigned ID " + assignedID);
			
			parameters.clear();

			// Get the list of clients from the server
			Vector serverlist = (Vector)client.execute ("getServerList", parameters);
			if (serverlist.size () == 0) {
				System.out.println("No servers registered at the master server!");
			} else {

				System.out.println("These servers are registered (by address):");
				
				for (int i = 0; i < serverlist.size(); i++) {
					String s = (String)serverlist.get(i);
					System.out.println("" + (i + 1) + ". " + s);
				}
			}

			// Start sending heartbeats
			boolean runclient = true;
			
			Vector hbparams = new Vector ();
			hbparams.add (assignedID);

			while (runclient) {
			  Thread.sleep(500);
				client.execute("heartbeat", hbparams);
			}

		} catch (Exception exception) {
			
			System.err.println ("Error running client! - "+exception);

		}
	}
}


		
