import org.apache.xmlrpc.*;
import java.util.Vector;

public class MasterServer {
	// The server instance
	private static WebServer rpcserver;
	private static MasterServer master;

	// Server list
	public Vector<ServerState> servers = new Vector<ServerState> ();
	public int nextServerId = 0;
	
	public static void main (String args[]) throws Exception {
		if (args.length < 1) {
	    System.err.println ("Usage: java MasterServer port");
	    System.exit (1);
		}
		int port = 0;
		try {
	    port = Integer.parseInt (args[0]);
		} catch (NumberFormatException nonumber) {
	    System.err.println ("Invalid port number: "+args[0]);
	    System.exit (1);
		}
		
		// Create the master server instance
		master = new MasterServer();

		// Run the RPC server
		rpcserver = new WebServer (port);
		rpcserver.addHandler ("$default", master);
		rpcserver.start ();

		System.out.println ("Startin master server...");
		System.out.println ("Listening on port " + port);

		boolean runserver = true;
		
		// Start looking for inactive clients
		while (runserver) {

			long time = System.currentTimeMillis ();
			
			for (int i = 0; i < master.servers.size (); i++) {
				ServerState s = master.servers.get (i);
				if (time > s.lastHeartbeat + 1000) {
					System.out.println ("Dropping client " + s.id + " (" + s.name + " at " + s.address + "). No heartbeat for one minute.");
					master.servers.remove (s);
				}
			}
		}
	}

	// Register a new server
	public int registerServer (String name, String address) {
		System.out.println ("Registering server " + name + " (at: " + address + ") with ID " + nextServerId);
		
		int assignedId = nextServerId;
		nextServerId++;

		ServerState newserver = new ServerState ();
		newserver.id = assignedId;
		newserver.name = name;
		newserver.address = address;
		newserver.lastHeartbeat = System.currentTimeMillis ();
		
		servers.add (newserver);
		
		return assignedId;
	}
	
	// Accept heartbeats from clients
	public boolean heartbeat (int id) {
		for (int i = 0; i < master.servers.size (); i++) {
			ServerState s = master.servers.get (i);
			if (s.id == id) {
				s.lastHeartbeat = System.currentTimeMillis();
			}
		}
		return true;
	}
	
	// Used by clients to get the server list
	public Vector getServerList () {
		Vector list = new Vector ();
		for (int i = 0; i < master.servers.size (); i++) {
			ServerState s = master.servers.get (i);
			list.add(s.address);
		}
		return list;
	}
	
}