/*
 The blackjack game server.
 */
import BlackJackApp.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POA;
import java.util.Properties;
import java.io.*;

class BlackJackImpl extends BlackJackPOA {
  private ORB orb;
	
  public void setORB(ORB orb_val) {
    orb = orb_val; 
  }
	
	Deck deckOfCards = new Deck ();
	Hand currentHand = new Hand ();
	int state = BlackJackService.STATE_WAITING;
	
	public BlackJackImpl () {

	}
	
	// Starts the game
	public void startGame () {
		deckOfCards = new Deck ();
		deckOfCards.shuffle ();
		currentHand = new Hand ();
		currentHand.addCard (deckOfCards.draw ());
		currentHand.addCard (deckOfCards.draw ());	
		state = BlackJackService.STATE_PLAYING;
	}
	
	// Returns the state of the game 
	public int getGameState () {
		return state;
	}
	
	// Get the current hand
	public HandStruct getHand () {
		HandStruct hs = new HandStruct ();
		hs.cards = new CardStruct [currentHand.cards.size ()];
		for (int i = 0; i < currentHand.cards.size (); i++) {
			Card c = currentHand.cards.elementAt (i);
			hs.cards[i] = new CardStruct (c.suit, c.rank);
		}
		return hs;
	}
	
	// Hit
	public void hit () {
		currentHand.addCard (deckOfCards.draw ());
		if (currentHand.getLowValue () > 21)
			state = BlackJackService.STATE_PLAYERBUSTED;
	}
	
	// Stand
	public void stand () {
		if (currentHand.getLowValue () > 21)
			state = BlackJackService.STATE_PLAYERBUSTED;
		else
			state = BlackJackService.STATE_PLAYERWON;
	}
	
	// Reset game
	public void resetGame () {
		state = BlackJackService.STATE_WAITING;
	}
	
	public void shutdownGame () {
    orb.shutdown (false);
  }
}


public class BlackJackServer {
	
  public static void main(String args[]) {

    try {
      ORB orb = ORB.init(args, null);
			
			POA rootpoa = POAHelper.narrow (orb.resolve_initial_references ("RootPOA"));
      rootpoa.the_POAManager ().activate ();
			
      BlackJackImpl gameImpl = new BlackJackImpl ();
      gameImpl.setORB (orb); 
			
      org.omg.CORBA.Object ref = rootpoa.servant_to_reference (gameImpl);
      BlackJack href = BlackJackHelper.narrow (ref);
			
      org.omg.CORBA.Object objRef = orb.resolve_initial_references("NameService");
      NamingContextExt ncRef = NamingContextExtHelper.narrow (objRef);
			
      String name = "BlackJackServer";
      NameComponent path[] = ncRef.to_name (name);
      ncRef.rebind (path, href);
			
			System.out.println ("Blackjack server started...");
			
      orb.run ();
    } 
		
		catch (Exception e) {
			System.err.println ("ERROR: " + e);
			e.printStackTrace (System.out);
		}
	  
		System.out.println ("Blackjack server stopped.");
		
  }
}
