/*
 The blackjack game server.
 */

import java.math.*;
import java.rmi.*;
import java.rmi.server.*;
import java.security.*;

public class BlackJackServer extends UnicastRemoteObject implements BlackJackService	{

	Deck deckOfCards = new Deck ();
	Hand currentHand = new Hand ();
	int state = BlackJackService.STATE_WAITING;
	
	public BlackJackServer () throws RemoteException {
		super();
	}

	// Starts the game
	public void startGame () throws RemoteException {
		deckOfCards = new Deck ();
		deckOfCards.shuffle ();
		currentHand = new Hand ();
		currentHand.addCard (deckOfCards.draw ());
		currentHand.addCard (deckOfCards.draw ());	
		state = BlackJackService.STATE_PLAYING;
	}
	
	// Returns the state of the game 
	public int getGameState () throws RemoteException {
		return state;
	}
	
	// Get the current hand
	public Hand getHand () throws RemoteException {
		return currentHand;
	}
	
	// Hit
	public void hit () throws RemoteException {
		currentHand.addCard (deckOfCards.draw ());
		if (currentHand.getLowValue () > 21)
			state = BlackJackService.STATE_PLAYERBUSTED;
	}
	
	// Stand
	public void stand () throws RemoteException {
		if (currentHand.getLowValue () > 21)
			state = BlackJackService.STATE_PLAYERBUSTED;
		else
			state = BlackJackService.STATE_PLAYERWON;
	}
	
	// Reset game
	public void resetGame () throws RemoteException {
		state = BlackJackService.STATE_WAITING;
	}
	
	public static void main ( String args[] ) throws Exception {

		System.setSecurityManager (new RMISecurityManager() {
			public void checkPermission (Permission perm) {}
		});

		BlackJackServer gameserver = new BlackJackServer();
	
		Naming.bind ("BlackjackServer", gameserver);
	
		System.out.println ("Blackjack server started...");
}
}